@@ -82,9 +82,6 @@ typedef struct {
 
 	GtkWindowGroup *window_group;
 	gboolean window_added;
-
-	/* Cached tree view entry for editing-canceled */
-	char *last_edited;
 } CEPageIP4Private;
 
 #define METHOD_COL_NAME 0
@@ -417,16 +414,11 @@ addr_add_clicked (GtkButton *button, gpointer user_data)
 	path = gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter);
 	column = gtk_tree_view_get_column (priv->addr_list, COL_ADDRESS);
 
-#if GTK_CHECK_VERSION(2,12,0)
-	cells = gtk_cell_layout_get_cells (GTK_CELL_LAYOUT (column));
-#else
-	cells = gtk_tree_view_column_get_cell_renderers (column);
-#endif
-
 	/* FIXME: using cells->data is pretty fragile but GTK apparently doesn't
 	 * have a way to get a cell renderer from a column based on path or iter
 	 * or whatever.
 	 */
+	cells = gtk_cell_layout_get_cells (GTK_CELL_LAYOUT (column));
 	gtk_tree_view_set_cursor_on_cell (priv->addr_list, path, column, cells->data, TRUE);
 
 	g_list_free (cells);
@@ -477,30 +469,6 @@ list_selection_changed (GtkTreeSelection *selection, gpointer user_data)
 		gtk_widget_set_sensitive (button, FALSE);
 }
 
-static void
-cell_editing_canceled (GtkCellRenderer *renderer, gpointer user_data)
-{
-	CEPageIP4 *self = CE_PAGE_IP4 (user_data);
-	CEPageIP4Private *priv = CE_PAGE_IP4_GET_PRIVATE (self);
-	GtkTreeModel *model = NULL;
-	GtkTreeSelection *selection;
-	GtkTreeIter iter;
-	guint32 column;
-
-	if (priv->last_edited) {
-		selection = gtk_tree_view_get_selection (priv->addr_list);
-		if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
-			column = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (renderer), "column"));
-			gtk_list_store_set (GTK_LIST_STORE (model), &iter, column, priv->last_edited, -1);
-		}
-
-		g_free (priv->last_edited);
-		priv->last_edited = NULL;
-
-		ce_page_changed (CE_PAGE (self));
-	}
-}
-
 static void
 cell_edited (GtkCellRendererText *cell,
              const gchar *path_string,
@@ -515,34 +483,10 @@ cell_edited (GtkCellRendererText *cell,
 	guint32 column;
 	GtkTreeViewColumn *next_col;
 
-	g_free (priv->last_edited);
-	priv->last_edited = NULL;
-
 	column = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (cell), "column"));
 	gtk_tree_model_get_iter (GTK_TREE_MODEL (store), &iter, path);
 	gtk_list_store_set (store, &iter, column, new_text, -1);
 
-	/* Try to autodetect the prefix from the given address if we can */
-	if (column == COL_ADDRESS && new_text && strlen (new_text)) {
-		char *prefix = NULL;
-		const char *guess_prefix = NULL;
-
-		gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), &iter, path_string);
-		gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, COL_PREFIX, &prefix, -1);
-		if (!prefix || !strlen (prefix)) {
-			if (!strncmp ("10.", new_text, 3))
-				guess_prefix = "8";
-			else if (!strncmp ("172.16.", new_text, 7))
-				guess_prefix = "16";
-			else if (!strncmp ("192.168.", new_text, 8))
-				guess_prefix = "24";
-
-			if (guess_prefix)
-				gtk_list_store_set (store, &iter, COL_PREFIX, guess_prefix, -1);
-		}
-		g_free (prefix);
-	}
-
 	/* Move focus to the next column */
 	column = (column >= COL_LAST) ? 0 : column + 1;
 	next_col = gtk_tree_view_get_column (priv->addr_list, column);
@@ -558,13 +502,11 @@ ip_address_filter_cb (GtkEntry *   entry,
                       const gchar *text,
                       gint         length,
                       gint *       position,
-                      gpointer     user_data)
+                      gpointer     data)
 {
-	CEPageIP4 *self = CE_PAGE_IP4 (user_data);
-	CEPageIP4Private *priv = CE_PAGE_IP4_GET_PRIVATE (self);
 	GtkEditable *editable = GTK_EDITABLE (entry);
 	int i, count = 0;
-	gchar *result = g_new0 (gchar, length);
+	gchar *result = g_new (gchar, length);
 
 	for (i = 0; i < length; i++) {
 		if ((text[i] >= '0' && text[i] <= '9') || (text[i] == '.'))
@@ -574,13 +516,11 @@ ip_address_filter_cb (GtkEntry *   entry,
 	if (count > 0) {
 		g_signal_handlers_block_by_func (G_OBJECT (editable),
 		                                 G_CALLBACK (ip_address_filter_cb),
-		                                 user_data);
+		                                 data);
 		gtk_editable_insert_text (editable, result, count, position);
-		g_free (priv->last_edited);
-		priv->last_edited = g_strdup (gtk_editable_get_chars (editable, 0, -1));
 		g_signal_handlers_unblock_by_func (G_OBJECT (editable),
 		                                   G_CALLBACK (ip_address_filter_cb),
-		                                   user_data);
+		                                   data);
 	}
 
 	g_signal_stop_emission_by_name (G_OBJECT (editable), "insert-text");
@@ -591,23 +531,17 @@ static void
 cell_editing_started (GtkCellRenderer *cell,
                       GtkCellEditable *editable,
                       const gchar     *path,
-                      gpointer         user_data)
+                      gpointer         data)
 {
-	CEPageIP4 *self = CE_PAGE_IP4 (user_data);
-	CEPageIP4Private *priv = CE_PAGE_IP4_GET_PRIVATE (self);
-
 	if (!GTK_IS_ENTRY (editable)) {
 		g_warning ("%s: Unexpected cell editable type.", __func__);
 		return;
 	}
 
-	g_free (priv->last_edited);
-	priv->last_edited = NULL;
-
 	/* Set up the entry filter */
 	g_signal_connect (G_OBJECT (editable), "insert-text",
 	                  (GCallback) ip_address_filter_cb,
-	                  user_data);
+	                  data);
 }
 
 static void
@@ -693,8 +627,7 @@ finish_setup (CEPageIP4 *self, gpointer unused, GError *error, gpointer user_dat
 	g_object_set (renderer, "editable", TRUE, NULL);
 	g_signal_connect (renderer, "edited", G_CALLBACK (cell_edited), self);
 	g_object_set_data (G_OBJECT (renderer), "column", GUINT_TO_POINTER (COL_ADDRESS));
-	g_signal_connect (renderer, "editing-started", G_CALLBACK (cell_editing_started), self);
-	g_signal_connect (renderer, "editing-canceled", G_CALLBACK (cell_editing_canceled), self);
+	g_signal_connect (renderer, "editing-started", G_CALLBACK (cell_editing_started), store);
 	priv->addr_cells[COL_ADDRESS] = GTK_CELL_RENDERER (renderer);
 
 	offset = gtk_tree_view_insert_column_with_attributes (priv->addr_list,
@@ -710,8 +643,7 @@ finish_setup (CEPageIP4 *self, gpointer unused, GError *error, gpointer user_dat
 	g_object_set (renderer, "editable", TRUE, NULL);
 	g_signal_connect (renderer, "edited", G_CALLBACK (cell_edited), self);
 	g_object_set_data (G_OBJECT (renderer), "column", GUINT_TO_POINTER (COL_PREFIX));
-	g_signal_connect (renderer, "editing-started", G_CALLBACK (cell_editing_started), self);
-	g_signal_connect (renderer, "editing-canceled", G_CALLBACK (cell_editing_canceled), self);
+	g_signal_connect (renderer, "editing-started", G_CALLBACK (cell_editing_started), store);
 	priv->addr_cells[COL_PREFIX] = GTK_CELL_RENDERER (renderer);
 
 	offset = gtk_tree_view_insert_column_with_attributes (priv->addr_list,
@@ -727,8 +659,7 @@ finish_setup (CEPageIP4 *self, gpointer unused, GError *error, gpointer user_dat
 	g_object_set (renderer, "editable", TRUE, NULL);
 	g_signal_connect (renderer, "edited", G_CALLBACK (cell_edited), self);
 	g_object_set_data (G_OBJECT (renderer), "column", GUINT_TO_POINTER (COL_GATEWAY));
-	g_signal_connect (renderer, "editing-started", G_CALLBACK (cell_editing_started), self);
-	g_signal_connect (renderer, "editing-canceled", G_CALLBACK (cell_editing_canceled), self);
+	g_signal_connect (renderer, "editing-started", G_CALLBACK (cell_editing_started), store);
 	priv->addr_cells[COL_GATEWAY] = GTK_CELL_RENDERER (renderer);
 
 	offset = gtk_tree_view_insert_column_with_attributes (priv->addr_list,
@@ -759,7 +690,10 @@ finish_setup (CEPageIP4 *self, gpointer unused, GError *error, gpointer user_dat
 }
 
 CEPage *
-ce_page_ip4_new (NMConnection *connection, GtkWindow *parent_window, GError **error)
+ce_page_ip4_new (NMConnection *connection,
+                 GtkWindow *parent_window,
+                 const char **out_secrets_setting_name,
+                 GError **error)
 {
 	CEPageIP4 *self;
 	CEPageIP4Private *priv;
@@ -805,10 +739,6 @@ ce_page_ip4_new (NMConnection *connection, GtkWindow *parent_window, GError **er
 	}
 
 	g_signal_connect (self, "initialized", G_CALLBACK (finish_setup), NULL);
-	if (!ce_page_initialize (parent, NULL, error)) {
-		g_object_unref (self);
-		return NULL;
-	}
 
 	return CE_PAGE (self);
 }
