@@ -17,21 +17,19 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * (C) Copyright 2007 - 2010 Red Hat, Inc.
+ * (C) Copyright 2007 Red Hat, Inc.
  */
 
-#include <config.h>
 #include <string.h>
 #include <netinet/ether.h>
 #include <glib.h>
-#include <glib/gi18n.h>
 
 #include <nm-device-ethernet.h>
 #include <nm-device-wifi.h>
+#include <nm-device-bt.h>
 #include <nm-gsm-device.h>
 #include <nm-cdma-device.h>
 #include <nm-access-point.h>
-#include <nm-settings.h>
 
 #include <nm-setting-connection.h>
 #include <nm-setting-wired.h>
@@ -41,6 +39,7 @@
 #include <nm-setting-gsm.h>
 #include <nm-setting-cdma.h>
 #include <nm-setting-pppoe.h>
+#include <nm-setting-bluetooth.h>
 #include <nm-utils.h>
 
 #include "utils.h"
@@ -64,6 +63,7 @@ static char *ignored_words[] = {
 	"adapter",
 	"[hex]",
 	"NDIS",
+	"Module",
 	NULL
 };
 
@@ -172,17 +172,12 @@ utils_get_device_description (NMDevice *device)
 
 	str = g_string_new_len (NULL, strlen (vendor) + strlen (product) + 1);
 
-	/* Another quick hack; if all of the fixed up vendor string
-	 * is found in product, ignore the vendor.
-	 */
-	if (!strcasestr (product, vendor)) {
-		g_string_append (str, vendor);
-		g_string_append_c (str, ' ');
-	}
+	g_string_append (str, vendor);
+	g_free (vendor);
 
+	g_string_append_c (str, ' ');
 	g_string_append (str, product);
 	g_free (product);
-	g_free (vendor);
 
 	description = str->str;
 	g_string_free (str, FALSE);
@@ -194,156 +189,6 @@ utils_get_device_description (NMDevice *device)
 	return description;
 }
 
-static GByteArray *
-file_to_g_byte_array (const char *filename)
-{
-	char *contents = NULL;
-	GByteArray *array = NULL;
-	gsize length = 0;
-
-	if (!g_file_get_contents (filename, &contents, &length, NULL))
-		return NULL;
-
-	array = g_byte_array_sized_new (length);
-	if (!array) {
-		g_free (contents);
-		return NULL;
-	}
-
-	g_byte_array_append (array, (unsigned char *) contents, length);
-	return array;
-}
-
-static gboolean
-fill_one_private_key (NMConnection *connection,
-                      const char *pk_tag,
-                      const char *pk_prop,
-                      const char *cc_prop)
-{
-	const char *filename;
-	NMSetting8021x *tmp;
-	NMSetting8021xCKType pk_type = NM_SETTING_802_1X_CK_TYPE_UNKNOWN;
-	gboolean need_client_cert = TRUE;
-
-	/* If the private key is PKCS#12, don't set the client cert */
-	filename = g_object_get_data (G_OBJECT (connection), pk_tag);
-	if (!filename)
-		return TRUE;
-
-	tmp = NM_SETTING_802_1X (nm_setting_802_1x_new ());
-	nm_setting_802_1x_set_private_key_from_file (tmp, filename, NULL, &pk_type, NULL);
-	if (pk_type == NM_SETTING_802_1X_CK_TYPE_PKCS12) {
-		GByteArray *array;
-
-		array = file_to_g_byte_array (filename);
-		if (array) {
-			NMSetting *s_8021x = nm_connection_get_setting (connection, NM_TYPE_SETTING_802_1X);
-
-			g_object_set (s_8021x,
-			              pk_prop, array,
-			              cc_prop, array,
-			              NULL);
-			g_byte_array_free (array, TRUE);
-			need_client_cert = FALSE;
-		}
-	}
-	g_object_unref (tmp);
-	return need_client_cert;
-}
-
-gboolean
-utils_fill_connection_certs (NMConnection *connection, GError **error)
-{
-	NMSetting8021x *s_8021x;
-	const char *filename;
-	GError *tmp_error = NULL;
-	gboolean need_client_cert = TRUE;
-
-	g_return_val_if_fail (connection != NULL, FALSE);
-
-	s_8021x = NM_SETTING_802_1X (nm_connection_get_setting (connection, NM_TYPE_SETTING_802_1X));
-	if (!s_8021x)
-		return TRUE;
-
-	filename = g_object_get_data (G_OBJECT (connection), NMA_PATH_CA_CERT_TAG);
-	if (filename) {
-		if (!nm_setting_802_1x_set_ca_cert_from_file (s_8021x, filename, NULL, &tmp_error)) {
-			g_set_error (error, tmp_error->domain, tmp_error->code,
-			             _("Could not read CA certificate: %s"), tmp_error->message);
-			g_clear_error (&tmp_error);
-			return FALSE;
-		}
-	}
-
-	/* If the private key is PKCS#12, don't set the client cert */
-	need_client_cert = fill_one_private_key (connection,
-	                                         NMA_PATH_PRIVATE_KEY_TAG,
-	                                         NM_SETTING_802_1X_PRIVATE_KEY,
-	                                         NM_SETTING_802_1X_CLIENT_CERT);
-	if (need_client_cert) {
-		filename = g_object_get_data (G_OBJECT (connection), NMA_PATH_CLIENT_CERT_TAG);
-		if (filename) {
-			if (!nm_setting_802_1x_set_client_cert_from_file (s_8021x, filename, NULL, &tmp_error)) {
-				g_set_error (error, tmp_error->domain, tmp_error->code,
-				             _("Could not read client certificate: %s"), tmp_error->message);
-				g_clear_error (&tmp_error);
-				return FALSE;
-			}
-		}
-	}
-
-	filename = g_object_get_data (G_OBJECT (connection), NMA_PATH_PHASE2_CA_CERT_TAG);
-	if (filename) {
-		if (!nm_setting_802_1x_set_phase2_ca_cert_from_file (s_8021x, filename, NULL, &tmp_error)) {
-			g_set_error (error, tmp_error->domain, tmp_error->code,
-			             _("Could not read inner CA certificate: %s"), tmp_error->message);
-			g_clear_error (&tmp_error);
-			return FALSE;
-		}
-	}
-
-	/* If the private key is PKCS#12, don't set the client cert */
-	need_client_cert = fill_one_private_key (connection,
-	                                         NMA_PATH_PHASE2_PRIVATE_KEY_TAG,
-	                                         NM_SETTING_802_1X_PHASE2_PRIVATE_KEY,
-	                                         NM_SETTING_802_1X_PHASE2_CLIENT_CERT);
-	if (need_client_cert) {
-		filename = g_object_get_data (G_OBJECT (connection), NMA_PATH_PHASE2_CLIENT_CERT_TAG);
-		if (filename) {
-			if (!nm_setting_802_1x_set_phase2_client_cert_from_file (s_8021x, filename, NULL, &tmp_error)) {
-				g_set_error (error, tmp_error->domain, tmp_error->code,
-				             _("Could not read inner client certificate: %s"), tmp_error->message);
-				g_clear_error (&tmp_error);
-				return FALSE;
-			}
-		}
-	}
-
-	return TRUE;
-}
-
-void
-utils_clear_filled_connection_certs (NMConnection *connection)
-{
-	NMSetting8021x *s_8021x;
-
-	g_return_if_fail (connection != NULL);
-
-	s_8021x = NM_SETTING_802_1X (nm_connection_get_setting (connection, NM_TYPE_SETTING_802_1X));
-	if (!s_8021x)
-		return;
-
-	g_object_set (s_8021x,
-	              NM_SETTING_802_1X_CA_CERT, NULL,
-	              NM_SETTING_802_1X_CLIENT_CERT, NULL,
-	              NM_SETTING_802_1X_PRIVATE_KEY, NULL,
-	              NM_SETTING_802_1X_PHASE2_CA_CERT, NULL,
-	              NM_SETTING_802_1X_PHASE2_CLIENT_CERT, NULL,
-	              NM_SETTING_802_1X_PHASE2_PRIVATE_KEY, NULL,
-	              NULL);
-}
-
-
 struct cf_pair {
 	guint32 chan;
 	guint32 freq;
@@ -550,10 +395,8 @@ utils_check_ap_compatible (NMAccessPoint *ap,
 	setting_bssid = nm_setting_wireless_get_bssid (s_wireless);
 	if (setting_bssid) {
 		struct ether_addr ap_addr;
-		const char *hw_addr;
 
-		hw_addr = nm_access_point_get_hw_address (ap);
-		if (hw_addr && ether_aton_r (hw_addr, &ap_addr)) {
+		if (ether_aton_r (nm_access_point_get_hw_address (ap), &ap_addr)) {
 			if (memcmp (setting_bssid->data, &ap_addr, ETH_ALEN))
 				return FALSE;
 		}
@@ -765,6 +608,66 @@ connection_valid_for_cdma (NMConnection *connection,
 	return TRUE;
 }
 
+static guint32
+get_connection_bt_type (NMConnection *connection)
+{
+	NMSettingBluetooth *s_bt;
+	const char *bt_type;
+
+	s_bt = (NMSettingBluetooth *) nm_connection_get_setting (connection, NM_TYPE_SETTING_BLUETOOTH);
+	if (!s_bt)
+		return NM_BT_CAPABILITY_NONE;
+
+	bt_type = nm_setting_bluetooth_get_connection_type (s_bt);
+	g_assert (bt_type);
+
+	if (!strcmp (bt_type, NM_SETTING_BLUETOOTH_TYPE_DUN))
+		return NM_BT_CAPABILITY_DUN;
+	else if (!strcmp (bt_type, NM_SETTING_BLUETOOTH_TYPE_PANU))
+		return NM_BT_CAPABILITY_NAP;
+
+	return NM_BT_CAPABILITY_NONE;
+}
+
+static gboolean
+connection_valid_for_bt (NMConnection *connection,
+                         NMSettingConnection *s_con,
+                         NMDevice *device,
+                         gpointer specific_object)
+{
+	NMSettingBluetooth *s_bt;
+	const GByteArray *array;
+	char *str;
+	const char *hw_addr;
+	int addr_match = FALSE;
+	guint32 bt_type;
+
+	if (strcmp (nm_setting_connection_get_connection_type (s_con), NM_SETTING_BLUETOOTH_SETTING_NAME))
+		return FALSE;
+
+	s_bt = NM_SETTING_BLUETOOTH (nm_connection_get_setting (connection, NM_TYPE_SETTING_BLUETOOTH));
+	if (!s_bt)
+		return FALSE;
+
+	array = nm_setting_bluetooth_get_bdaddr (s_bt);
+	if (!array || (array->len != ETH_ALEN))
+		return FALSE;
+
+	bt_type = get_connection_bt_type (connection);
+	if (!(bt_type & nm_device_bt_get_capabilities (NM_DEVICE_BT (device))))
+		return FALSE;
+
+	str = g_strdup_printf ("%02X:%02X:%02X:%02X:%02X:%02X",
+	                       array->data[0], array->data[1], array->data[2],
+	                       array->data[3], array->data[4], array->data[5]);
+	hw_addr = nm_device_bt_get_hw_address (NM_DEVICE_BT (device));
+	if (hw_addr)
+		addr_match = !g_ascii_strcasecmp (hw_addr, str);
+	g_free (str);
+
+	return addr_match;
+}
+
 gboolean
 utils_connection_valid_for_device (NMConnection *connection,
                                    NMDevice *device,
@@ -787,6 +690,8 @@ utils_connection_valid_for_device (NMConnection *connection,
 		return connection_valid_for_gsm (connection, s_con, device, specific_object);
 	else if (NM_IS_CDMA_DEVICE (device))
 		return connection_valid_for_cdma (connection, s_con, device, specific_object);
+	else if (NM_IS_DEVICE_BT (device))
+		return connection_valid_for_bt (connection, s_con, device, specific_object);
 	else
 		g_warning ("Unknown device type '%s'", g_type_name (G_OBJECT_TYPE(device)));
 
@@ -851,22 +756,6 @@ utils_ether_ntop (const struct ether_addr *mac)
 }
 
 
-static void
-add_one_name (gpointer data, gpointer user_data)
-{
-	NMExportedConnection *exported = NM_EXPORTED_CONNECTION (data);
-	NMConnection *connection;
-	NMSettingConnection *s_con;
-	const char *id;
-	GSList **list = (GSList **) user_data;
-
-	connection = nm_exported_connection_get_connection (exported);
-	s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION));
-	id = nm_setting_connection_get_id (s_con);
-	g_assert (id);
-	*list = g_slist_append (*list, (gpointer) id);
-}
-
 char *
 utils_next_available_name (GSList *connections, const char *format)
 {
@@ -874,10 +763,16 @@ utils_next_available_name (GSList *connections, const char *format)
 	char *cname = NULL;
 	int i = 0;
 
-	g_slist_foreach (connections, add_one_name, &names);
+	for (iter = connections; iter; iter = g_slist_next (iter)) {
+		NMConnection *candidate = NM_CONNECTION (iter->data);
+		NMSettingConnection *s_con;
+		const char *id;
 
-	if (g_slist_length (names) == 0)
-		return g_strdup_printf (format, 1);
+		s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (candidate, NM_TYPE_SETTING_CONNECTION));
+		id = nm_setting_connection_get_id (s_con);
+		g_assert (id);
+		names = g_slist_append (names, (gpointer) id);
+	}	
 
 	/* Find the next available unique connection name */
 	while (!cname && (i++ < 10000)) {
@@ -901,6 +796,48 @@ utils_next_available_name (GSList *connections, const char *format)
 	return cname;
 }
 
+char *
+utils_hash_ap (const GByteArray *ssid,
+               NM80211Mode mode,
+               guint32 flags,
+               guint32 wpa_flags,
+               guint32 rsn_flags)
+{
+	unsigned char input[66];
+
+	memset (&input[0], 0, sizeof (input));
+
+	if (ssid)
+		memcpy (input, ssid->data, ssid->len);
+
+	if (mode == NM_802_11_MODE_INFRA)
+		input[32] |= (1 << 0);
+	else if (mode == NM_802_11_MODE_ADHOC)
+		input[32] |= (1 << 1);
+	else
+		input[32] |= (1 << 2);
+
+	/* Separate out no encryption, WEP-only, and WPA-capable */
+	if (  !(flags & NM_802_11_AP_FLAGS_PRIVACY)
+	    && (wpa_flags == NM_802_11_AP_SEC_NONE)
+	    && (rsn_flags == NM_802_11_AP_SEC_NONE))
+		input[32] |= (1 << 3);
+	else if (   (flags & NM_802_11_AP_FLAGS_PRIVACY)
+	         && (wpa_flags == NM_802_11_AP_SEC_NONE)
+	         && (rsn_flags == NM_802_11_AP_SEC_NONE))
+		input[32] |= (1 << 4);
+	else if (   !(flags & NM_802_11_AP_FLAGS_PRIVACY)
+	         &&  (wpa_flags != NM_802_11_AP_SEC_NONE)
+	         &&  (rsn_flags != NM_802_11_AP_SEC_NONE))
+		input[32] |= (1 << 5);
+	else
+		input[32] |= (1 << 6);
+
+	/* duplicate it */
+	memcpy (&input[33], &input[0], 32);
+	return g_compute_checksum_for_data (G_CHECKSUM_MD5, input, sizeof (input));
+}
+
 typedef struct {
 	const char *tag;
 	const char *replacement;
